import { DIDAttribute, IOperator, IUpdateData, PubKeyType } from '@ew-did-registry/did-resolver-interface';
import { KeyType } from '@ew-did-registry/keys';
import { BigNumber } from 'ethers';
import { IDIDDocumentFull } from './interface';
import { DIDDocumentLite } from '../lite';
declare class DIDDocumentFull extends DIDDocumentLite implements IDIDDocumentFull {
    private _operator;
    constructor(did: string, operator: IOperator);
    /**
     * Creates new empty DID document
     *
     * @example
     * ```typescript
     *  import { DIDDocumentFull } from '@ew-did-registry/did-document';
     *
     *  const document = new DIDDocumentFull(did, operator);
     *  await document.create();
     * ```
     * @return { boolean }
     */
    create(): Promise<boolean>;
    /**
     * Deactivates DID document
     *
     * @example
     * ```typescript
     * import { DIDDocumentFull } from '@ew-did-registry/did-document';
     *
     * const document = new DIDDocumentFull(did, operator);
     * await document.create();
     * await document.update(didAttribute, updateData, validity);
     * await document.deactivate();
     * ```
     * @return { boolean }
     */
    deactivate(): Promise<void>;
    /**
     * Updates attribute on the DID document
     *
     * @example
     * ```typescript
     * import { DIDDocumentFull } from '@ew-did-registry/did-document';
     * import { DIDAttribute, Algorithms, PubKeyTypes } from '@ew-did-registry/did-document';
     *
     * const document = new DIDDocumentFull(did, operator);
     * await document.create();
     * const didAttribute = DIDAttribute.PublicKey;
     * const validity = 5 * 60 * 1000;
     * await document.update(
     *  didAttribute,
     *  {
     *    type: PubKeyType.VerificationKey2018,
     *    value: new Keys().publicKey,
     *  },
     *  validity,
     *  );
     * ```
     * @param { DIDAttribute } attribute
     * @param { IUpdateData } data
     * @param { number } validity - time in milliseconds during the attribute will be valid
     * if missing it should be set to Number.MAX_SAFE_INTEGER by the operator
     * @return { boolean }
     */
    update(attribute: DIDAttribute, data: IUpdateData, validity?: number): Promise<BigNumber>;
    revokeDelegate(delegateType: PubKeyType, delegateDID: string): Promise<boolean>;
    revokeAttribute(attributeType: DIDAttribute, updateData: IUpdateData): Promise<boolean>;
    updatePublicKey({ publicKey, algo, type, tag, validity, }: {
        publicKey: string;
        did?: string;
        algo: KeyType;
        type: PubKeyType;
        tag: string;
        validity?: number;
    }): Promise<BigNumber>;
    updateDelegate({ delegatePublicKey, algo, type, validity, }: {
        delegatePublicKey: string;
        algo: KeyType;
        type: PubKeyType;
        validity?: number;
    }): Promise<BigNumber>;
}
export default DIDDocumentFull;
